/*************************************************************
 * Analóg hőmérő, NEMO-Q KAT120B kijelzővel
 * 
 * A hőmérsékletet MCP9700 analóg hőmérővel mérjük.  Az MCP9700 
 * hőmérő nullapontja = 500 mV, érzékenysége = 10 mV/C.
 * Az analóg jelet az A4 analóg bementre kötjük. 
 * Az MCP9700 hőmérő jelét a beépített 1.1 V-os referenciához 
 * képest mérjük, az eredményt Celsius fokokra átszámítjuk.
 * A statisztikus ingadozás csökkentés érdekében
 * minden ciklusban 1100 mérést végzünk és átlagolunk. 
 *
 * A KAT120B kijelző TAKT (órajel) és DATA (adat) bemeneteit 
 * egy-egy NPN tranzisztor segítségével hajtjuk meg, ezek bázisát
 * vezérli az Arduino D3 és D4 kimenete. A kijelző hátoldali 
 * címválasztó kapcsolói mind ON állásban legyenek!
 *************************************************************/

#define CLKPIN  3 
#define DATAPIN 4  
long mysum;
uint16_t i, mv;
uint8_t d1, d2, d3;

void setup() {
  analogReference(INTERNAL); // 1.1 V-os referencia
  pinMode(CLKPIN, OUTPUT);   // Órajel kimenet 
  pinMode(DATAPIN, OUTPUT);  // Adat kimenet
  digitalWrite(DATAPIN,LOW); // Adat kimenent alaphelyzetbe
  digitalWrite(CLKPIN,LOW);  // Órajel kimenet alaphelyzetbe
}

void loop() {
  mysum = 0;
  for(int i=0; i<1100; i++) {
    mysum += analogRead(A4);
  }
  mv = mysum>>10;          // Osztás 1024-gyel
  mv = mv - 500;           // Nullapont
  if(mv > 999) mv = 999;
  d1 = mv/100;             // Százasok
  d2 = mv/10%10;           // Tízesek
  d3 = mv%10;              // Egyesek
  myshiftOut(d1<<4);       // Egyesek kiküldése
  myshiftOut((d2<<4)+d3);  // Tízesek és százasok kiküldése
  myshiftOut(B01111110);   // Kódkapcsoló állása
  myshiftOut(B11111111);   // Üzenet vége jelzés és szinkronizálás
  delay(2000);    
}  

// Egy bájt kiküldése bitenként (MSBFIRST mód)
void myshiftOut(uint8_t val) {
  uint8_t i;
  for (i = 0; i < 8; i++)  {
    digitalWrite(DATAPIN,bitRead(val,7-i)); // Adatvonal beállítása
    delayMicroseconds(10);    
    digitalWrite(CLKPIN, HIGH);             // Órajel aktíválás
    delayMicroseconds(10);
    digitalWrite(CLKPIN, LOW);	            // Órajel alaphelyzetbe
    delayMicroseconds(10);
  }
}







